#!/usr/bin/env python
# Time-stamp: <qtdotter.py 2015-05-19 12:49:55 Mark Voorhies>

# qt4 matplotlib imports based on
# http://matplotlib.org/examples/user_interfaces/embedding_in_qt4_wtoolbar.html

import numpy as np
from matplotlib.figure import Figure
from matplotlib.backend_bases import key_press_handler
from matplotlib.backends.backend_qt4agg import (
    FigureCanvasQTAgg as FigureCanvas,
    NavigationToolbar2QT as NavigationToolbar)
from matplotlib.backends import qt4_compat
use_pyside = qt4_compat.QT_API == qt4_compat.QT_API_PYSIDE

if use_pyside:
    from PySide.QtCore import *
    from PySide.QtGui import *
else:
    from PyQt4.QtCore import *
    from PyQt4.QtGui import *

# qt4 widgets modified from
# http://matplotlib.org/examples/user_interfaces/embedding_in_qt4_wtoolbar.html

class AppForm(QMainWindow):
    def __init__(self, fig, parent=None):
        QMainWindow.__init__(self, parent)
        self.create_main_frame(fig)
        self.on_draw()

    def create_main_frame(self, fig):
        self.main_frame = QWidget()

        self.fig = fig
        self.canvas = FigureCanvas(self.fig)
        self.canvas.setParent(self.main_frame)
        self.canvas.setFocusPolicy(Qt.StrongFocus)
        self.canvas.setFocus()

        self.mpl_toolbar = NavigationToolbar(self.canvas, self.main_frame)

        self.canvas.mpl_connect('key_press_event', self.on_key_press)

        vbox = QVBoxLayout()
        vbox.addWidget(self.canvas)  # the matplotlib canvas
        vbox.addWidget(self.mpl_toolbar)
        self.main_frame.setLayout(vbox)
        self.setCentralWidget(self.main_frame)

    def on_draw(self):
        self.canvas.draw()

    def on_key_press(self, event):
        # implement the default mpl key press events described at
        # http://matplotlib.org/users/navigation_toolbar.html#navigation-keyboard-shortcuts
        key_press_handler(event, self.canvas, self.mpl_toolbar)

from Dotplot import Dotplot, NucleotideDotplot, alpha_aa, blosum62
from DotplotQt import QtDotMediator
from FastaFile import FastaFile

import re

if(__name__ == "__main__"):
    import sys
    from optparse import OptionParser
    parser = OptionParser(
        usage = "usage: %prog [options] hseq.fasta vseq.fasta")

    parser.add_option(
        "-W", "--window", dest = "window",
        help = "window size",
        type = "int",
        default = 25)

    (options, args) = parser.parse_args()

    if(len(args) < 2):
        parser.print_help()
        sys.exit(1)

    if(options.window < 1):
        sys.stderr.write("Window size must be positive\n")
        sys.exit(1)

    hseq = FastaFile(open(args[0])).seqs.values()[0].upper()
    vseq = FastaFile(open(args[1])).seqs.values()[0].upper()

    if(re.search("^[ATUGCN]+$", hseq+vseq)):
        print "Calculating nucleotide dotplot..."
        dotplot = NucleotideDotplot(hseq, vseq, options.window)
    else:
        print "Calculating protein dotplot..."
        dotplot = Dotplot(hseq, vseq, alpha_aa, blosum62, options.window)

    print "Rendering..."

    app = QApplication(sys.argv)
    dp = QtDotMediator(dotplot)
    form = AppForm(dp.fig)
    form.show()
    app.exec_()

    
